/*
*  ASLicenseValidator.h
*  ActivationTest
*
*  Created by Ken Sillart on 2/25/09.
*  Copyright 2009 MYOB Technology Pty. Ltd.. All rights reserved.
*
*/

#pragma once

#include "CrossPlatform/CrossPlatform.h"
#include "ASObject.h"

namespace ASFoundation
{
	class EXTERNAL_API ASLicenseValidator : public ASObject
	{
	public:
		enum 
		{
			kMachineIdNotFoundErrorId = 1,
			kEmptyLicenseFileErrorId,
			kUnableToSafeLicenseFileErrorId
		};
		virtual ~ASLicenseValidator(void);

		static ASLicenseValidator* Create(const CPString& PublicKey);

		// Validating license data/files
		bool IsLicenseDictionaryValid(const CPDictionary& LicenseDictionary);
		virtual bool IsLicenseDataValid(const CPData& LicenseData);
		virtual bool IsLicenseFileValid(const CPString& licenseFilePath);

		// Generating dictionary from license data/files
		virtual CPDictionary CreateDictionaryForLicenseData(const CPData& LicenseData);
		virtual CPDictionary CreateDictionaryForLicenseFile(const CPString& LicenseFilePath) = 0;

		// Generating license data/files
		virtual CPData CreateLicenseDataFromDictionary(const CPDictionary& LicenseDictionary) = 0;
		CPError CreateLicenseFileFromDictionary(const CPDictionary& LicenseDictionary, const CPString& LicenseFilePath);
		CPError CreateLicenseFileFromData(const CPData& LicenseData, const CPString& LicenseFilePath);

		CPString GetPublicKey(void) const;

		CPString GetMachineId(void) const;
		virtual bool DictionaryHasMatchingMachineId(const CPDictionary& LicenseDictionary) const;	
		virtual bool HasMatchingMachineId(const CPString& MachineId) const;

	protected:
		ASLicenseValidator(void);
		ASLicenseValidator(const CPString& PublicKey);

		void SetPublicKey(const CPString& PublicKey);

		CPString ConvertMACAddressToMachineId(const CPString& MACAddress) const;

		static CPString BufferToHexString(const void* Buffer, const unsigned int Length);

	private:
		CPString m_PublicKey;
	};
}

