////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPArray.h
//    Cross platform array class.
//
//  Copyright (C) 2009 MYOB US, Inc., all rights reserved.
//
//  Change History (most recent first):
//
//	10/20/09  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPUInteger.h"
#include "CrossPlatform/CPObject.h"

class CPBool;
class CPString;
class CPValue;
class CPDate;
class CPData;
class CPDictionary;
class CPNumber;

class EXTERNAL_API CPArray : public CPObject
{
public:
	static CPString ClassType(void);

	CPArray(void);
	CPArray(const CPArray& Array);
	CPArray(const CPObject& Array);

	~CPArray(void);

	const CPArray& operator=(const CPArray& Array);
	//	const CPArray& operator=(CPId Array);

	CPObject* operator[](int Index) const; // caller owns the object

	bool IsEmpty(void) const;
	CPUInteger Count(void) const;

	CPObject* ObjectAtIndex(CPUInteger Index) const; // caller owns this object
	CPArray ArrayAtIndex(CPUInteger Index) const;
	CPData DataAtIndex(CPUInteger Index) const;
	CPDictionary DictionaryAtIndex(CPUInteger Index) const;
	CPDate DateAtIndex(CPUInteger Index) const;
	CPNumber NumberAtIndex(CPUInteger Index) const;
	CPBool BooleanAtIndex(CPUInteger Index) const;
	CPString StringAtIndex(CPUInteger Index) const;
	CPValue ValueAtIndex(CPUInteger Index) const;

	void SetObjectAtIndex(const CPObject& Object, CPUInteger Index);
	void AddObject(const CPObject& Object);
	void RemoveAllObjects(void);
	void RemoveLastObject(void);
	void RemoveObjectAtIndex(CPUInteger Index);

	void ExchangeObjectAtIndexWithObjectAtIndex(CPUInteger Object1Index, CPUInteger Object2Index);

	CPString ComponentsJoinedByString(const CPString& Separator) const;

#ifdef __OBJC__
	// Define methods that will be needed only when bridging to Cocoa
	CPArray(NSArray* Array);
	
	const CPArray& operator=(NSArray* Array);
	
	operator NSArray*(void) const;
	bool operator==(NSArray* Array) const;
	bool operator!=(NSArray* Array) const;
	
#endif  // #ifdef __OBJC__
#ifdef __COREFOUNDATION_CFBASE__
	// Define methods that will be needed only when bridging to CoreFoundation
	CPArray(CFArrayRef String);
	const CPArray& operator=(CFArrayRef Array);

	operator CFArrayRef(void) const;
	//bool operator==(CFArrayRef Array) const;
	//bool operator!=(CFArrayRef Array) const;
#endif

protected:
	CPId InternalObjectAtIndex(CPUInteger Index) const;
};
