////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPBundle.h
//    Cross-platform bundle class. A bundle is a logical grouping for a code and its associated resources.
//
//  Copyright (C) 2009 MYOB US, Inc., all rights reserved.
//
//  Change History (most recent first):
//
//	10/20/09  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

class CPString;

class EXTERNAL_API CPBundle : public CPObject
{
public:
	static CPString ClassType(void);
	static const CPBundle& MainBundle(void);
	static const CPString& DefaultStringsTableName(void);

	CPBundle(void);
	CPBundle(const CPBundle& Bundle);
	//CPBundle(CPId Bundle);

	virtual ~CPBundle(void);

	const CPBundle& operator=(const CPBundle& Bundle);
	const CPBundle& operator=(CPId Bundle);

	CPString BundleIdentifier(void) const;
	CPString BundlePath(void) const;
	CPString ExecutablePath(void) const;
	CPString ResourcePath(void) const;

	CPString LocalizedStringForKey(const CPString& Key) const;
	CPString LocalizedStringForKeyInTableNamed(const CPString& Key, const CPString& TableName) const;
	CPString LocalizedStringForKeyWithDefaultValue(const CPString& Key, const CPString& DefaultValue) const;
	CPString LocalizedStringForKeyWithDefaultValueInTableNamed(const CPString& Key, const CPString& DefaultValue, const CPString& TableName) const;
	
	CPString PathForResourceOfType(CPString ResourceName, CPString Extension);


#ifdef __OBJC__
	CPBundle(const NSBundle* Bundle);
	const CPBundle& operator=(const NSBundle* Bundle);
#else
protected:
	CPBundle(CPString BundlePath);

	CPDictionary StringTableWithPath(const CPString& TablePath) const;

private:
	CPString m_ExecutablePath;
	CPString m_BundlePath;
	CPString m_ResourcePath;

	void SetBundlePath(CPString BundlePath);
#endif  // #ifdef __OBJC__
};
