////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPData.h
//    Cross-platform string class.
//
//  Copyright (C) 2009-2010 MYOB US, Inc., all rights reserved.
//
//  Change History (most recent first):
//
//	10/20/09  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPUInteger.h"
#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPRange.h"
#include "CrossPlatform/CPStringEncoding.h"
#include "CrossPlatform/CPString.h"
#include "CrossPlatform/CPDictionary.h"

class EXTERNAL_API CPData : public CPObject
{
public:
	static CPString ClassType(void);

	static CPData DataWithLength(CPUInteger Length);
	static CPData DataWithContentsOfFile(CPString FilePath);
	static CPData DataWithBytes(const void* Bytes, CPUInteger Length);
	static CPData DataWithStringUsingEncoding(CPString String, CPStringEncoding Encoding);

	CPData(void);
	CPData(const CPData& Data);
	CPData(CPString String);
	CPData(CPDictionary Dictionary);
	CPData(const CPObject& Data);
	CPData(CPId Data);

	~CPData(void);

	CPData SubdataWithRange(const CPRange& Range) const;
	void InitWithLength(CPUInteger Length);
	void InitWithData(CPData Data);
	void InitWithString(CPString String);
	void InitWithStringUsingEncoding(CPString String, CPStringEncoding Encoding);
	void InitWithContentsOfFile(CPString FilePath);
	void InitWithDictionary(CPDictionary Dictionary);
	void InitWithBytes(const void* Bytes, CPUInteger Length);

	const CPData& operator=(CPData Data);
	const CPData& operator=(CPId Data);
	const CPData& operator=(CPString String);
	const CPData& operator=(CPDictionary Dictionary);

	bool IsEmpty(void) const;
	void Empty(void);

	CPUInteger Length(void) const;
	CPRange Range(void) const;

	const void* Bytes(void) const;
	void* MutableBytes(void) const;
	const char* CharPointer(void) const;
	const wchar_t* WideCharPointer(void) const;
	const void GetBytes(void* ToBuffer) const;
	const void GetBytesWithLength(void* ToBuffer, CPUInteger Length) const;
	const void GetBytesWithRange(void* ToBuffer, CPRange Range) const;


	bool WriteToFile(CPString FilePath) const;
	bool WriteToFileAtomically(CPString FilePath) const;

	void AppendData(CPData Apppend);
	void AppendString(CPString Apppend);
	void AppendBytes(const void* Append, CPUInteger ByteCount);
	
#ifdef __OBJC__
	// Define methods that will be needed only when bridging to Cocoa
	CPData(NSData* Data);
	
	const CPData& operator=(NSData* Data);
	
	operator NSData*() const;
	
#endif  // #ifdef __OBJC__
	
};
