////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPDateFormatter.h
//    Cross-platform date class.
//
//  Copyright (C) 2009-2010 MYOB US, Inc., all rights reserved.
//
//  Change History (most recent first):
//
//  06/09/10  G.R.W.  Added method DateFromMonthDayYearHourMinuteSecond and SetFromMonthDayYearHourMinuteSecond.
//  03/25/09  K.L.S.  Created file.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
 @header CPDateFormatter
 CPDateFormatter objects are use to convert dates to and from strings.
 */

#pragma once

#include "CPObject.h"
#include "CPString.h"

/*!
 @class CPDateFormat
 @abstract CPDateFormat provide commonly used format strings to be used to convert to and from strings.
*/
class EXTERNAL_API CPDateFormat
{
public:
	static CPString ddMMyyyyDateFormat(void);
	static CPString MMddyyyyDateFormat(void);
	static CPString MMddyyyyHHmmssSSSDateFormat(void);
	static CPString yyyyMMddDateFormat(void);
	static CPString HHmmssDateFormat(void);
	static CPString HHmmssSSSDateFormat(void);
	static CPString HHmmDateFormat(void);
};

/*!
 @class CPDateFormatter
 @abstract CPDateFormatter convert dates to and from strings.
 To parse strings containing dates and to generate string representations of a date, you should use an instance of CPDateFormatter using the
 methods DateFromString() and StringFromDate() respectively.
*/

class EXTERNAL_API CPDateFormatter : public CPObject
{
public:
	enum Enum_CPDateFormatterStyle
	{	
		CPDateFormatterNoStyle = 0,
		CPDateFormatterShortStyle = 1,
		CPDateFormatterMediumStyle = 2,
		CPDateFormatterLongStyle = 3,
		CPDateFormatterFullStyle = 4
	};

	typedef int CPDateFormatterStyle;

	static CPDateFormatter CurrentLocaleDateFormatter(void);
	static CPDateFormatter DefaultFormatter(void);
	static CPDateFormatter DateFormatterUsingDateFormat(CPString dateFormat);
	static CPDateFormatter DateFormatterUsingDateFormatDateStyle(CPString dateFormat, CPDateFormatterStyle dateStyle);
	static CPDateFormatter DateFormatterUsingDateFormatDateStyleTimeStyle(CPString dateFormat, CPDateFormatterStyle dateStyle, 
		CPDateFormatterStyle timeStyle);

	CPDateFormatter(void);
	CPDateFormatter(const CPDateFormatter& DateFormatter);
	CPDateFormatter(CPId Id);
	~CPDateFormatter(void);

	const CPDateFormatter& operator=(const CPDateFormatter& DateFormatter);
	const CPDateFormatter& operator=(CPId DateFormatter);

	void SetDateFormat(CPString DateFormat);

	/*!
	 @method     DateFromString (class method)
	 @abstract   Returns an NSDate object with a date or date and time value specified by a given string in the string representation format
	 (MM-DD-YYYY HH:MM:SS) or (MM-DD-YYYY).
	 @discussion You can use this object when you want a specific date and time components.
	 @param      DateString  A string that specifies a date or a date and time value.
	 @result     A CPDate object representing the date and time specified.
	 */
	CPDate DateFromString(const CPString& DateString) const;

	/*!
	 @method     DateFromStringUsingFormat (class method)
	 @abstract   Returns an NSDate object with a date or date and time value specified by a given string in the format specified.
	 @discussion You can use this object when you want a specific date and time components.
	 @param      DateString  A string that specifies a date or a date and time value.
	 @param      DateFormat  A string that specifies a format that the date string is in.
	 @result     A CPDate object representing the date and time specified.
	 */
//	CPDate DateFromStringUsingFormat(const CPString& DateString, const CPString& DateFormat const);


	// No format methods use mm-dd-yyyy format. As this default could cause problems UI code should already convert strings to native date object. (ie: NSDate)
	// May need to look back at this decision if it proves to be to much of a headache for others to deal with
	CPString StringFromDate(const CPDate& Date) const;
//	CPString StringFromDateUsingFormat(const CPDate& Date, const CPString& DateFormat) const;
	//CPString LocalizedDateTimeStringFromDate(const CPDate& Date);
	//CPString LocalizedDateStringFromDate(const CPDate& Date);
	//CPString LocalizedTimeStringFromDate(const CPDate& Date);

private:
	CPDateFormatter(CPDateFormatterStyle DateStyle, CPDateFormatterStyle TimeStyle);
	CPString m_DF;
};
