////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPDebugLog.h
//  ************* Put a discription of the file here. *********************
//
//  Copyright (C) 2008 MYOB Technology Pty. Ltd., all rights reserved.
//
//  Change History (most recent first):
//
//	7/25/08  K.L.S.  Created File.
//
//!!!: Define DEBUG in the Xcode project setting debug build.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*!
 @header CPDebugLog
 @abstract   Allows output of messages to the console in a debug build.
 @copyright 2008 MYOB Technology Pty Ltd., all rights reserved.
 @updated 2008-07-26
 @discussion Functions and Macros used to output to the console log. These will output nothing in a release build.
*/

#pragma once

#if ! defined(NDEBUG) && ! defined(_DEBUG)
#warning "when including DebugLog.h you should define NDEBUG or _DEBUG. Defaulting to defining NDEBUG.
#define NDEBUG
#endif

class CPString;


/*!
    @function
    @abstract   Output a message to the console log.
    @discussion Will output a message to the console log if running a debug build, will do nothing in a release build. Only works in debug build when DEBUG defined.
    @param      format Message to be sent to the console log. It will perform parameter replacement on string.
    @result     none
*/

/*!
 @defined 
 @abstract   Outputs a message to the console.
 @param      format format message to be sent to the console log. It will perform parameter replacement on string.
 @param      ... the replacement parameters.
 @discussion Will output a message to the console log if running a debug build, will do nothing in a release build.
 */
#if ! defined(_DEBUG) && ! defined(NDEBUG)
#error //make sure yout define either _DEBUG or NDEBUG
#endif
void EXTERNAL_API DebugLog(CPString format, ...);

/*!
 @defined 
 @abstract   Outputs a message to the console with info about where in the code the message happened.
 @param      s format message to be sent to the console log. It will perform parameter replacement on string.
 @param      ... the replacement parameters.
 @discussion Will output a message to the console log if running a debug build, will do nothing in a release build.
 Info includes routine name, file name, and line number. Only works in debug build when DEBUG defined.
 */
#ifdef _MAC
#define TRACE_LOG(s, ...) DebugLog((s "    Function:%s File:%@ Line:%d"), ## __VA_ARGS__, __func__,  [@__FILE__ lastPathComponent], __LINE__);
#else
#define TRACE_LOG(s, ...) DebugLog((s "    Function:%s File:%@ Line:%d"), ## __VA_ARGS__, __func__,  CPSTR(__FILE__).LastPathComponent().Ref(), __LINE__);
#endif
