////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPDictionary.h
//    Cross platform dictionary class.
//
//  Copyright (C) 2009 MYOB US, Inc., all rights reserved.
//
//  Change History (most recent first):
//
//	10/20/09  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPUInteger.h"
#include "CrossPlatform/CPObject.h"

class CPBool;
class CPString;
class CPArray;
class CPData;
class CPEnumerator;
class CPDate;
class CPNumber;

class EXTERNAL_API CPDictionary : public CPObject
{
public:
	static CPString ClassType(void);

	static CPDictionary DictionaryWithContentsOfFile(const CPString& FilePath);

	CPDictionary(void);
	CPDictionary(const CPDictionary& Dictionary);
	CPDictionary(const CPData& Data);
	CPDictionary(CPId Data);

	~CPDictionary(void);

	const CPDictionary& operator=(const CPDictionary& Dictionary);
	const CPDictionary& operator=(CPId Dictionary);
	const CPDictionary& operator=(const CPData& Data);
	const CPDictionary& operator+=(const CPDictionary& Dictionary);

	bool IsEmpty(void) const;

	CPUInteger Count(void) const;

	void SetValueForKey(const CPObject& Value, const CPString& Key);
	void SetObjectForKey(const CPObject& Value, const CPString& Key);
	CPId InternalValueForKey(const CPString& Key) const;
	CPString ObjectsClassTypeForKey(const CPString& Key) const;
	CPObject* ValueForKey(const CPString& Key) const; // caller owns this object
	CPObject* ValueOfClassTypeForKey(const CPString& ClassTypeId, const CPString& Key) const; // caller owns this object

	void RemoveObjectForKey(const CPString& Key);

	CPString StringForKey(const CPString& Key) const;
	CPArray ArrayForKey(const CPString& Key) const;
	CPDictionary DictionaryForKey(const CPString& Key) const;
	CPData DataForKey(const CPString& Key) const;
	CPDate DateForKey(const CPString& Key) const;
	CPBool BooleanForKey(const CPString& Key) const;
	CPNumber NumberForKey(const CPString& Key) const;
//	CPValue ValueForKey(const CPString& Key) const;

	void AddDictionary(const CPDictionary& AdditionalDictionary);

	CPEnumerator KeyEnumerator(void) const;

	bool WriteToFile(const CPString& FilePath) const;
	bool WriteToFileAtomically(const CPString& FilePath) const;

#ifdef __OBJC__
	// Define methods that will be needed only when bridging to Cocoa
	CPDictionary(NSDictionary* Dictionary);
	CPDictionary(CFDictionaryRef Dictionary);

	const CPDictionary& operator=(NSDictionary* Dictionary);
	const CPDictionary& operator=(CFDictionaryRef Dictionary);
	const CPDictionary& operator+=(NSDictionary* Dictionary);
	const CPDictionary& operator+=(CFDictionaryRef Dictionary);

	operator NSDictionary*(void) const;
	operator CFDictionaryRef(void) const;

#endif  // #ifdef __OBJC__

protected:
	bool CPIdForKeyIfPresent(const CPString& Key, CPId& Value) const;
};
