/*
*  CPError.h
*  CrossPlatformFramework
*
*  Created by Ken Sillart on 3/4/09.
*  Copyright 2009 MYOB Technology Pty. Ltd.. All rights reserved.
*
*/

#pragma once

#include "CrossPlatform/CPObject.h"

class CPString;
class CPDictionary;
class CPArray;

class EXTERNAL_API CPError : public CPObject
{
public:
	static CPString ClassType(void);
	static CPString LocalizedDescriptionKey(void);
	static CPString LocalizedRecoverySuggestionErrorKey(void);
	static CPString LocalizedRecoveryOptionsErrorKey(void);
	
	static CPError ErrorWithCodeDescriptionAndDomain(int Code, const CPString& ErrorDescription, const CPString& Domain);

	CPError(void);
	CPError(const CPError& OtherObject);
	CPError(CPId OtherObject);

	~CPError(void);

	bool IsError(void) const;

	const CPError& operator=(const CPError& OtherObject);
	const CPError& operator=(CPId OtherObject);

	int Code(void) const;
	CPString Domain(void) const;
	CPDictionary UserInfo(void) const;

	CPObject* ValueForKey(const CPString& Key) const; // caller owns this object
	void SetValueForKey(const CPObject& Value, const CPString& Key);

	virtual CPString Description(void) const;

#ifdef __OBJC__
	CPError(NSError* Error);
	const CPError& operator=(NSError* Error);
	operator NSError*() const;	
#endif  // #ifdef __OBJC__
};

//#define MakeCPErrorWithRecoveryAndFileAsDomain(errNo,errDesc,recoverDesc,recoverOpt) MakeCPError(errNo,errDesc,recoverDesc,recoverOpt, FilePathToDomain(__FILE__))
//#define MakeCPErrorWithFileAsDomain(errNo,errDesc,recoverDesc) MakeCPError(errNo,errDesc,recoverDesc, FilePathToDomain(__FILE__))

#define FileNameAsCPErrorDomain() FilePathToDomain(__FILE__)

CPString EXTERNAL_API FilePathToDomain(const CPString& FilePath);
CPError EXTERNAL_API MakeCPError(int Code, const CPString& ErrorDescription, const CPString& RecoveryDescription, const CPArray& RecoveryOptions, const CPString& Domain);
CPError EXTERNAL_API MakeCPError(int Code, const CPString& ErrorDescription, const CPString& RecoveryDescription, const CPDictionary& AdditionalInfo, const CPString& Domain);
CPError EXTERNAL_API MakeCPError(int Code, const CPString& ErrorDescription, const CPString& RecoveryDescription, const CPString& Domain);
//CPError MakeCPError(int Code, const CPString& ErrorDescription, const CPString& Domain);

CPError EXTERNAL_API MakeCPError(int Code, const CPString& Domain, const CPDictionary& UserInfo); // Base

//#define MakeCPErrorWithCodehDomainAndDescription(n,domain,desc) \
//[NSError errorWithDomain:domain code:n userInfo:[NSDictionary dictionaryWithObjectsAndKeys:desc, NSLocalizedDescriptionKey, nil]]
//#define MakeCPErrorWithCodeAndDescription(n,desc) \
//[NSError errorWithDomain:[NSString stringWithFormat:@"%s",typeid(this).name] code:n userInfo:[NSDictionary dictionaryWithObjectsAndKeys:desc, NSLocalizedDescriptionKey, nil]]
//#define MakeCPErrorWithCode(n) \
//[NSError errorWithDomain:[NSString stringWithFormat:@"%s",typeid(this).name] code:n userInfo:nil]
//
//}
