///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPFileManager.h
//    Cross-platform file manager.
//
//  Copyright (C) 2009-2010 MYOB US, Inc., all rights reserved.
//
//  Change History (most recent first):
//
//	01/03/10  K.L.S.  Added DisplayNameAtPath.
//  12/14/09  G.R.W.  Added the DirectoryContentsAtPath method.
//	03/11/09  K.L.S.  Created File.
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

//namespace CrossPlatform
//{
	class CPArray;
	class CPString;
	class CPDictionary;

	class EXTERNAL_API CPFileManager
	{
	public:	
		static bool ItemExistsAtPath(const CPString& Path);
		static bool ItemExistsAtPath(const CPString& Path, bool& IsDirectory);
		static bool IsReadableItemAtPath(const CPString& Path);
		static bool IsWritableItemAtPath(const CPString& Path);
		static bool IsExecutableItemAtPath(const CPString& Path);
		static bool IsDeletableItemAtPath(const CPString& Path);
		static bool IsDirectory(const CPString& Path);

		static bool CopyPathToPath(CPString FromPath, CPString ToPath);
		static bool MovePathToPath(CPString FromPath, CPString ToPath);
		static bool RemoveItemAtPath(const CPString& Path);
		static bool CreateDirectoryAtPath(const CPString& Path);
		static bool CreateDirectoryAtPathWithAttributes(const CPString& Path, const CPDictionary& Attributes);

		static CPArray DirectoryContentsAtPath(const CPString& Path);
		static CPArray DirectoryContentsAtPathWithMatchingExtensions(const CPString& Path, const CPArray& Extensions);

		static CPString DisplayNameAtPath(CPString Path);

		static CPString TempDirectoryPath(void);

	private:
		CPFileManager(void);
		~CPFileManager(void);
	};
//}
