////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPId.h
//    Cross-platform base class.
//
//  Copyright (C) 2010 MYOB US, Inc., all rights reserved.
//
//  Change History (most recent first):
//
//	07/03/10  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*!
@header CPId
@abstract  Base class for most "CP" classes. Provides some very basic functionality.
@copyright 2010 MYOB US, Inc., all rights reserved.
@updated 2010-07-03
@discussion
*/

#pragma once

#ifdef __OBJC__

#include <Cocoa/Cocoa.h>

#define CPNil nil

typedef NSObject CPNativeObject;
typedef id CPId;

#else  // #ifdef __OBJC__

//typedef NSObject CPNativeObject;

#include "ExternalAPIMacros.h"

struct EXTERNAL_API CPId
{
	const void* id;
	CPId(void) { id = 0; }
	CPId(const CPId& P) { id = P.id; }
	const CPId& operator=(const CPId& P) { id = P.id; return *this; }

	bool operator!=(const CPId& P) const { return id != P.id; }
	bool operator!=(const void* P) const { return id != P; }
	bool operator==(const void* P) const { return id == P; }
	bool operator==(const CPId& P) const { return (this == &P) || (id == P.id); }
	operator const void*(void) const { return id; }
};
typedef CPId CPId;

#ifndef NULL
#ifdef __cplusplus
#define NULL    0
#else
#define NULL    ((void*)0)
#endif
#endif

//typedef const void* CPId;
typedef const struct __CPNull * CPNullRef;
EXTERNAL_API CPId kcfNullRef(void);

#define CPNil kcfNullRef()

#endif  // #else  // #ifdef __OBJC__
