///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPJsonTranslator.h
//    Json Parser & Encoder converts from Json to a CPObject and vice versa.
//
//  Copyright (C) 2010 MYOB US, Inc., all rights reserved.
//
//  Change History (most recent first):
//
//	02/22/10  G.R.W.  Minor changes.
//	02/21/10  K.L.S.  Added this file into the CrossPlatform Library.
//	01/20/10  G.R.W.  Initial creation of file based on the test file that came with the jsoncpp library.
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPString.h"
#include "CrossPlatform/CPError.h"

class EXTERNAL_API CPJsonTranslator
{
public:
	CPJsonTranslator(void);
	CPJsonTranslator(const CPJsonTranslator& Translator);
	virtual ~CPJsonTranslator(void); 

	const CPJsonTranslator& operator=(const CPJsonTranslator& Translator);

	//Parse the Json encoded string and return a CPObject 
	CPObject* Parse(const CPString& InputJsonString);

	//Parse the Json file and return a CPObject
	CPObject* ParseFromFilePath(const CPString& InputFilePath);

	//Make a Json string from the cpDictionary
	CPString Encode(const CPObject& Object) const;

	//Make a Json file from the cpDictionary using the previously set outputFilePath
	CPError EncodeToFilePath(const CPObject& Object, const CPString& OutputFilePath);

private:
	CPString ReadInputFilePath(const CPString& FilePath);

	CPObject* ParseNext(void* value);

	
	void* EncodeNext(const CPObject& Object) const;
};
