////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPObject.h
//    Cross-platform base class.
//
//  Copyright (C) 2009-2010 MYOB US, Inc., all rights reserved.
//
//  Change History (most recent first):
//
//	10/20/09  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*!
@header CPObject
@abstract  Base class for most "CP" classes. Provides some very basic functionality.
@copyright 2009-2010 MYOB US, Inc., all rights reserved.
@updated 2010-07-03
@discussion CPObject is the base class for most CrossPlatform classes. It provides some basic functionality.
*/

#pragma once

#include "ExternalApiMacros.h"
#include "CPId.h"
#include "CPUInteger.h"

class CPString;

class EXTERNAL_API CPObject
{
public:
	static CPString ClassType(void); // you must create one in any subclass of CPObject

	static CPObject* Create(CPId Object);

	CPObject(void);
	CPObject(const CPObject& OtherObject);

	virtual ~CPObject(void);

	CPString ObjectsClassType(void) const;
	bool IsTypeOfClass(const CPString& ClassTypeId) const;
	bool IsSameClassTypeAs(const CPObject& Object) const;

	const CPObject& operator=(const CPObject& OtherObject);

	bool operator==(const CPObject& OtherObject);
	bool operator!=(const CPObject& OtherObject);

	virtual bool IsEmpty(void) const;
	virtual void Empty(void);

	virtual CPString Description(void) const;

	virtual void Retain(void);
	virtual void Release(void);
	CPUInteger RetainCount(void) const;

//	virtual operator CPId(void) const;

	CPId Ref(void) const;
	const CPId* InternalRef(void) const;

protected:
	CPId m_Ref;

	CPObject(const CPId OtherObject);
	const CPObject& operator=(CPId OtherObject);
	bool operator==(CPId OtherObject);
	bool operator!=(CPId OtherObject);
};
