////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPRange.h
//  Cross-platform range class. A range is something that has a starting point and and a length.
//
//  Copyright (C) 2009 MYOB US, Inc., all rights reserved.
//
//  Change History (most recent first):
//
//	6/2/09  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPUInteger.h"

class EXTERNAL_API CPRange
{
public:
	static CPRange RangeWithLocationAndLength(CPUInteger Location, CPUInteger Length);

	CPRange(void);
	CPRange(const CPRange& Range);
	CPRange(CPUInteger Location, CPUInteger Length);

	void InitWithLocationAndLength(CPUInteger Location, CPUInteger Length);
	void InitWithRange(const CPRange& Range);

	const CPRange& operator=(const CPRange& Range);
	bool operator==(const CPRange& Range) const;

	void Empty(void);

	bool IsEmpty(void) const;

	CPUInteger Location(void) const;
	CPUInteger Length(void) const;
	CPUInteger EndOfRange(void) const;
	bool IsLocationInRange(CPUInteger Location) const;
	bool IsEqualToRange(const CPRange& Range) const;

	// A range covering all indices in and between itself and Range.
	// If one range is completely contained in the other, the returned range is equal to the larger range.
	CPRange UnionRange(const CPRange& Range) const;

	// A range describing the intersection of itself and Range  that is, a range containing the indices that exist in both ranges.
	// If the returned ranges length field is 0, then the two ranges dont intersect, and the value of the location field is undefined.
	CPRange IntersectionRange(const CPRange& Range) const;

	// A range describing the remainder of itself starting at the end of Range  that is, a range containing the indices that exist in
	// this range once you move the location to Range.EndOfRange().
	// If the returned ranges length field is 0, then nothing else exists, and the value of the location field is undefined.
	CPRange DifferenceRange(const CPRange& Range) const;
	// Look to have DifferenceRange renamed to RangeRemainingAfterRange
	//CPRange RangeRemainingAfterRange(const CPRange& Range) const;

#ifdef __OBJC__
	CPRange(const NSRange& Range);
	const CPRange& operator=(const NSRange& Range);

	void InitWithNSRange(const NSRange& Range);

	operator NSRange(void) const;

	bool operator==(const NSRange& Range) const;

	bool IsEqualToRange(const NSRange& Range) const;

	CPRange UnionRange(const NSRange& Range) const;
	CPRange IntersectionRange(const NSRange& Range) const;
#endif  // #ifdef __OBJC__

private:
	CPUInteger m_Location;
	CPUInteger m_Length;
};
