////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPURL.h
//    Cross-platform URL class.
//
//  Copyright (C) 2009 MYOB US, Inc., all rights reserved.
//
//  Change History (most recent first):
//
//	10/20/09  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPObject.h"

class EXTERNAL_API CPURL : public CPObject
{
public:
	static CPString ClassType(void);
	static CPURL URLFromFilePath(const CPString& FilePath);

	CPURL(void);
	CPURL(const CPURL& URL);
	CPURL(const CPString& AbsoluteURL);
	CPURL(const CPURL& BaseURL, const CPString& RelativeURL);
	CPURL(CPId URL);

	~CPURL(void);

	const CPURL& operator=(const CPURL& URL);
	const CPURL& operator=(const CPString& URL);
	const CPURL& operator=(CPId URL);

	const bool operator==(const CPURL& URL);
	const bool operator!=(const CPURL& URL);

	// Path routines
	CPURL URLByAppendingPathComponent(const CPString& PathComponent) const;
	CPURL URLByAppendingPathExtension(const CPString& PathComponent) const;
	CPURL URLByDeletingLastPathComponent(void) const;
	CPURL URLByDeletingPathExtension(void) const;

	CPString AbsoluteString(void) const;
	CPURL AbsoluteURL(void) const;
	CPURL BaseURL(void) const;
	CPString RelativeString(void) const;
	CPURL RelativePath(void) const;
	CPString LastPathComponent(void) const;
	CPString PathExtension(void) const;
};
