///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPUserDefaults.h
//    Cross platform user defaults class.
//
//  Copyright (C) 2010 MYOB US, Inc., all rights reserved.
//
//  Change History (most recent first):
//
//	06/08/10  K.L.S.  Created File.
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPArray.h"
#include "CrossPlatform/CPData.h"
#include "CrossPlatform/CPDictionary.h"
#include "CrossPlatform/CPString.h"
#include "CrossPlatform/CPNumber.h"

class EXTERNAL_API CPUserDefaults : public CPObject
{
public:
	static CPUserDefaults StandardUserDefaults(void);
	static CPUserDefaults StandardAllUsersDefaults(void);

	virtual ~CPUserDefaults(void);

	CPObject* ObjectForKey(CPString DefaultsKey) const; // caller owns object
	void SetObjectForKey(const CPObject& DefaultsValue, CPString DefaultsKey);
	CPArray ArrayForKey(CPString DefaultsKey) const;
	bool BoolForKey(CPString DefaultsKey) const;
	void SetBoolForKey(bool DefaultsValue, CPString DefaultsKey);
	CPData DataForKey(CPString DefaultsKey) const;
	CPDate DateForKey(CPString DefaultsKey) const;
	CPDictionary DictionaryForKey(CPString DefaultsKey) const;
	CPArray StringArrayForKey(CPString DefaultsKey) const;
	CPString StringForKey(CPString DefaultsKey) const;
	CPNumber NumberForKey(CPString DefaultsKey) const;
	int IntegerForKey(CPString DefaultsKey) const;
	void SetIntegerForKey(int DefaultsValue, CPString DefaultsKey);
	float FloatForKey(CPString DefaultsKey) const;
	void SetFloatForKey(float DefaultsValue, CPString DefaultsKey);
	double DoubleForKey(CPString DefaultsKey) const;
	void SetDoubleForKey(double DefaultsValue, CPString DefaultsKey);
	bool RemoveObjectForKey(CPString DefaultsKey) const;

	bool RemoveUserDefaults(void) const;

	bool AccessingAllUsersDefaults(void) const;
	bool Synchronize(void) const;

protected:
	CPUserDefaults(void);
	CPUserDefaults(bool AccessAllUsersDefaults);

private:
	CPString m_BundleIdentifier;
	bool m_AccessAllUsersDefaults;
};
