////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  OBImage.h
//      Definition of OBImage class. Class is cross-platform handling an image. This is for class for use with obsolete drawing.
//
//  Copyright: (C) 2010 MYOB US, Inc., all rights reserved.
//
//  Change History (most recent first): 
//
//  10/10/10  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/PlatformSpecificMacros.h"
#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPData.h"
#include "CrossPlatform/CPString.h"
#include "ObsoleteBridge/OBSize.h"
#include "ObsoleteBridge/OBRect.h"
#ifdef _WINDOWS
#include <Windows.h>
#include <GDIplus.h>
#endif

class EXTERNAL_API OBImage : public CPObject
{
public:
	static OBImage ImageWithContentsOfFile(CPString Path);
	static OBImage ImageWithData(CPData ImageData);

	OBImage(void);
	virtual ~OBImage(void);

	OBImage(const OBImage& ImageIn);
	const OBImage& operator=(const OBImage& ImageIn);
	
	void Release(void);

	CPData Data(void) const;
	void InitWithImageData(CPData ImageData);
	void InitWithContentsOfFile(CPString Path);
	
	OBSize Size(void) const;

#ifdef __OBJC__
	OBImage(NSImage* ImageIn);
	const OBImage& operator=(NSImage* ImageIn);
	CPData m_AlternateData;
#else
	OBImage(const Gdiplus::Image* ImageIn);
	const OBImage& operator=(const Gdiplus::Image* ImageIn);
	operator Gdiplus::Image*(void) const;
#endif
};
