////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  OBRect.h
//    A typedef file for types used in the CrossPlatform function calls.
//
//  Copyright (C) 2010 MYOB US, Inc., all rights reserved.
//
//  Change History (most recent first):
//
//	09/16/10  G.R.W.  Added Ken's changes
//	09/14/10  G.R.W.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/ExternalAPIMacros.h"
#include "ObsoleteBridge/OBPoint.h"
#include "ObsoleteBridge/OBSize.h"

class EXTERNAL_API OBRect
{
public:
	OBPoint origin;
	OBSize size;

	static OBRect EmptyRect(void);
	static OBRect RectWithOriginAndSize(OBPoint Origin, OBSize Size);
	static OBRect RectWithXYWidthAndHeight(OBCoordinate X, OBCoordinate Y, OBCoordinate Width, OBCoordinate Height);
	
	OBRect(void);
	OBRect(OBPoint Origin, OBSize Size);
	OBRect(const OBRect& RectIn);
	virtual ~OBRect(void);
	
	const OBRect& operator=(const OBRect& RectIn);
	
	OBCoordinate MaxX(void) const;
	OBCoordinate MaxY(void) const;
	OBCoordinate MidX(void) const;
	OBCoordinate MidY(void) const;
	OBCoordinate MinX(void) const;
	OBCoordinate MinY(void) const;
	OBCoordinate Width(void) const;
	OBCoordinate Height(void) const;

	bool IsEqualToRect(OBRect Rect) const;
	bool IsEmpty(void) const;

	OBRect InsetRect(OBCoordinate DX, OBCoordinate DY) const;
	OBRect OffsetRect(OBCoordinate DX, OBCoordinate DY) const;
	OBRect IntegralRect(void) const;
	OBRect UnionRect(OBRect Rect) const;
	OBRect IntersectionRect(OBRect Rect) const;
	//	OBRect DivideRect(OBRect* slice, OBRect* rem, OBCoordinate amount, OBRectEdge edge) const;
	bool PointInRect(OBPoint Point) const;
	bool ContainsRect(OBRect Rect) const;
	bool IntersectsRect(OBRect Rect) const;

#ifdef __OBJC__
	OBRect(const NSRect& Rect);
	const OBRect& operator=(const NSRect& Rect);
	operator NSRect(void) const;
#endif

protected:
	OBRect(OBCoordinate X, OBCoordinate Y, OBCoordinate Width, OBCoordinate Height);
};
